latin_lower = 'abcdefghijklmnopqrstuvwxyz'
latin_upper = latin_lower.upper()
latin = latin_lower + latin_upper
latin_36 = '0123456789' + latin_lower
latin_36_human = latin_36.replace('0', '').replace('o', '')
